==> vais_speech-1.0.7.tar.gz: android/src/main/java/tsyliveaudiorecord/live_audio_record/AudioRecorder.java <==
package tsyliveaudiorecord.live_audio_record;

import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.util.Log;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Queue;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicLong;

import io.reactivex.BackpressureStrategy;
import io.reactivex.Flowable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Consumer;
import io.reactivex.processors.BehaviorProcessor;
import io.reactivex.processors.PublishProcessor;
import io.reactivex.schedulers.Schedulers;
import io.reactivex.subscribers.DisposableSubscriber;

public class AudioRecorder implements IAudioRecorder {

    private static final int RECORDER_STATE_FAILURE = -1;
    private static final int RECORDER_STATE_IDLE = 0;
    private static final int RECORDER_STATE_STARTING = 1;
    private static final int RECORDER_STATE_STOPPING = 2;
    private static final int RECORDER_STATE_BUSY = 3;

    private volatile int recorderState;

    private int mRecorderSampleRate = 16000;

    private AudioSaveHelper audioSaveHelper = null;

    private final Object recorderStateMonitor = new Object();

//    private byte[] recordBuffer;
    private long mDataSize = 0;

    private CompositeDisposable compositeDisposable ;
    Queue<HashMap<String, Object>> qe;

    private final AtomicLong mRecordTimeCounter = new AtomicLong(0);
    private final AtomicBoolean mIsPaused = new AtomicBoolean(false);
    private RecordTime currentRecordTime;
    private double mPeakPower = -120;
    private double mAveragePower = -120;
    private String mStatus = "unset";



    public AudioRecorder(AudioSaveHelper audioSaveHelper) {
        this.audioSaveHelper = audioSaveHelper;
        this.qe = new LinkedList();
        mStatus = "initialized";
        resetRecorder();
//        recordBuffer = null;
        Log.d("kiemtra","new Audio record");
        compositeDisposable = new CompositeDisposable();

    }

    @SuppressWarnings("ResultOfMethodCallIgnored") private void onRecordFailure() {
        recorderState = RECORDER_STATE_FAILURE;
        finishRecord();
    }

    @Override public void startRecord(int recorderSampleRate,String filePath) {

        if (recorderState != RECORDER_STATE_IDLE) {
            return;
        }
        this.mRecorderSampleRate = recorderSampleRate;
        mStatus = "recording";

        audioSaveHelper.setSampleRate(mRecorderSampleRate);
        audioSaveHelper.setFilePath(filePath);
        startTimer();
        recorderState = RECORDER_STATE_STARTING;
        startRecordThread();
    }

    private final BehaviorProcessor<RecordTime> recordTimeProcessor = BehaviorProcessor.create();

    private void startTimer() {
        getTimerObservable().subscribeOn(Schedulers.newThread()).subscribe(recordTimeProcessor);
    }

    private Flowable<RecordTime> getTimerObservable() {
        return Flowable.interval(1000, TimeUnit.MILLISECONDS)
                .filter(timeElapsed -> !mIsPaused.get())
                .map(tick -> {
                    long seconds = mRecordTimeCounter.incrementAndGet();
                    RecordTime recordTime = new RecordTime();
                    recordTime.millis = seconds * 1000;
                    recordTime.hours = seconds / (60 * 60);
                    seconds = seconds % (60 * 60);
                    recordTime.minutes = seconds / 60;
                    seconds = seconds % 60;
                    recordTime.seconds = seconds;
                    currentRecordTime = recordTime;
                    return recordTime;
                });
    }


    private Flowable<byte[]> audioDataFlowable ;

    private final PublishProcessor<byte[]> recordDataPublishProcessor = PublishProcessor.create();

    private void startRecordThread() {

        audioDataFlowable =  Flowable.create(emitter -> {
            int bufferSize = AudioRecord.getMinBufferSize(mRecorderSampleRate, AudioFormat.CHANNEL_IN_MONO, AudioFormat.ENCODING_PCM_16BIT);

            AudioRecord recorder = new AudioRecord(MediaRecorder.AudioSource.VOICE_RECOGNITION, mRecorderSampleRate,
                    Constants.RECORDER_CHANNELS, Constants.RECORDER_AUDIO_ENCODING, bufferSize);
            audioSaveHelper.createNewFile();
            Log.d("kiemtra","new AudioRecord");

            try {
                if (recorderState == RECORDER_STATE_STARTING) {
                    recorderState = RECORDER_STATE_BUSY;
                }
                recorder.startRecording ();
                Log.d("kiemtra3423423423423","startRecording");


                do {
                    if (!mIsPaused.get()) {
                        byte[] recordBuffer = new byte[bufferSize];
                        int bytesRead = recorder.read(recordBuffer, 0, bufferSize);
                        if (bytesRead == 0) {
                            Log.e(AudioRecorder.class.getSimpleName(), "error: " + bytesRead);
                            onRecordFailure();
                        }else{
                            Log.d("kiemtra4235345",""+bytesRead);
                            emitter.onNext(recordBuffer);

                        }
                    }
                } while (recorderState == RECORDER_STATE_BUSY);
            } finally {
                recorder.stop();
                recorder.release();
                Log.d("kiemtra","release nay");
            }
            emitter.onComplete();
        }, BackpressureStrategy.DROP);
        audioDataFlowable.subscribeOn(Schedulers.io()).subscribe(recordDataPublishProcessor);
        compositeDisposable.add(recordDataPublishProcessor.onBackpressureBuffer()
                .observeOn(Schedulers.io())
                .subscribeWith(new DisposableSubscriber<byte[]>() {
                    @Override public void onNext(byte[] bytes) {
                        mDataSize += bytes.length;
                        updatePowers(bytes);



                        HashMap<String, Object> currentResult = new HashMap<>();
                        currentResult.put("duration", getDuration() * 1000);
                        currentResult.put("path", "");
                        currentResult.put("audioFormat", "mExtension");
                        currentResult.put("peakPower",mPeakPower);
                        currentResult.put("averagePower", mAveragePower);
                        currentResult.put("isMeteringEnabled", true);
                        currentResult.put("status", mStatus);
                        currentResult.put("buffer", bytes);

                        qe.add(currentResult);

                        Log.d("kiemtra",qe.size() + "");

                        audioSaveHelper.onDataReady(bytes);
                    }

                    @Override public void onError(Throwable t) {

                    }

                    @Override public void onComplete() {
                        audioSaveHelper.onRecordingStopped(currentRecordTime);
                        synchronized (recorderStateMonitor) {
                            recorderState = RECORDER_STATE_IDLE;
                            recorderStateMonitor.notifyAll();
                        }
                    }
                }));
    }

    @Override public void finishRecord() {
        int recorderStateLocal = recorderState;
        if (recorderStateLocal != RECORDER_STATE_IDLE) {
            synchronized (recorderStateMonitor) {
                recorderStateLocal = recorderState;
                if (recorderStateLocal == RECORDER_STATE_STARTING
                        || recorderStateLocal == RECORDER_STATE_BUSY) {

                    recorderStateLocal = recorderState = RECORDER_STATE_STOPPING;
                }

                do {
                    try {
                        if (recorderStateLocal != RECORDER_STATE_IDLE) {
                            recorderStateMonitor.wait();
                        }
                    } catch (InterruptedException ignore) {
                        /* Nothing to do */
                    }
                    recorderStateLocal = recorderState;
                } while (recorderStateLocal == RECORDER_STATE_STOPPING);
            }
        }
        Log.d("kiemtra","finishRecord");


        resetRecorder();
        compositeDisposable.dispose();
        compositeDisposable = null;
        audioDataFlowable = null;
    }

    @Override public void pauseRecord() {
        qe.clear();
        mStatus = "paused";
        mPeakPower = -120;
        mAveragePower = -120;
        mIsPaused.set(true);
    }

    @Override public void resumeRecord() {
        mIsPaused.set(false);
        mStatus = "recording";

    }

    @Override public boolean isRecording() {
        return recorderState != RECORDER_STATE_IDLE;
    }

    public Flowable<byte[]> getAudioDataFlowable() {
        return recordDataPublishProcessor;
    }

    public Disposable subscribeTimer(Consumer<RecordTime> timerConsumer) {
        Disposable disposable =
                recordTimeProcessor.observeOn(AndroidSchedulers.mainThread()).subscribe(timerConsumer);
        compositeDisposable.add(disposable);
        return disposable;
    }

    boolean isPaused() {
        return mIsPaused.get();
    }

    public static class RecordTime {
        public long seconds = 0;
        public long minutes = 0;
        public long hours = 0;
        long millis = 0;
    }

    private int getDuration(){
        long duration = mDataSize / (mRecorderSampleRate * 2 * 1);
        return (int)duration;
    }


    private short[] byte2short(byte[] bData) {
        short[] out = new short[bData.length/2];
        ByteBuffer.wrap(bData).order(ByteOrder.LITTLE_ENDIAN).asShortBuffer().get(out);
        return out;
    }

    private void resetRecorder(){
        mPeakPower = -120;
        mAveragePower = -120;
        mDataSize = 0;
    }

    private void updatePowers(byte[] bdata){
        short[] data = byte2short(bdata);
        short sampleVal = data[data.length - 1];
        String[] escapeStatusList = new String[]{"paused", "stopped", "initialized", "unset"};

        if(sampleVal == 0 || Arrays.asList(escapeStatusList).contains(mStatus)){
            mAveragePower = -120; // to match iOS silent case
        }
        else {
            // iOS factor : to match iOS power level
            double iOSFactor = 0.25;
            mAveragePower = 20 * Math.log(Math.abs(sampleVal) / 32768.0)  * iOSFactor;
        }

        mPeakPower = mAveragePower;
        // Log.d(LOG_NAME, "Peak: " + mPeakPower + " average: "+ mAveragePower);
    }
}

==> vais_speech-1.0.8.tar.gz: android/src/main/java/tsyliveaudiorecord/live_audio_record/AudioRecorder.java <==
package tsyliveaudiorecord.live_audio_record;

import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.util.Log;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Arrays;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Queue;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicLong;

import io.reactivex.BackpressureStrategy;
import io.reactivex.Flowable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Consumer;
import io.reactivex.processors.BehaviorProcessor;
import io.reactivex.processors.PublishProcessor;
import io.reactivex.schedulers.Schedulers;
import io.reactivex.subscribers.DisposableSubscriber;

public class AudioRecorder implements IAudioRecorder {

    private static final int RECORDER_STATE_FAILURE = -1;
    private static final int RECORDER_STATE_IDLE = 0;
    private static final int RECORDER_STATE_STARTING = 1;
    private static final int RECORDER_STATE_STOPPING = 2;
    private static final int RECORDER_STATE_BUSY = 3;

    private volatile int recorderState;

    private int mRecorderSampleRate = 16000;

    private AudioSaveHelper audioSaveHelper = null;

    private final Object recorderStateMonitor = new Object();

//    private byte[] recordBuffer;
    private long mDataSize = 0;

    private CompositeDisposable compositeDisposable ;
    Queue<HashMap<String, Object>> qe;

    private final AtomicLong mRecordTimeCounter = new AtomicLong(0);
    private final AtomicBoolean mIsPaused = new AtomicBoolean(false);
    private final AtomicBoolean mIsStopped = new AtomicBoolean(false);


    private RecordTime currentRecordTime;
    private double mPeakPower = -120;
    private double mAveragePower = -120;
    private String mStatus = "unset";

    private Boolean _isWriteFile = false;




    public AudioRecorder(AudioSaveHelper audioSaveHelper) {
        this.audioSaveHelper = audioSaveHelper;
        this.qe = new LinkedList();
        mStatus = "initialized";
        resetRecorder();
//        recordBuffer = null;
        Log.d("kiemtra","new Audio record");
        compositeDisposable = new CompositeDisposable();

    }

    @SuppressWarnings("ResultOfMethodCallIgnored") private void onRecordFailure() {
        recorderState = RECORDER_STATE_FAILURE;
        finishRecord();
    }

    @Override public void startRecord(int recorderSampleRate,String filePath,Boolean isWriteFile) {
        this._isWriteFile = isWriteFile;

        if (recorderState != RECORDER_STATE_IDLE) {
            return;
        }
        this.mRecorderSampleRate = recorderSampleRate;
        mStatus = "recording";

        if(_isWriteFile){
            audioSaveHelper.setSampleRate(mRecorderSampleRate);
            audioSaveHelper.setFilePath(filePath);
        }

        startTimer();
        recorderState = RECORDER_STATE_STARTING;
        startRecordThread();
    }

    private final BehaviorProcessor<RecordTime> recordTimeProcessor = BehaviorProcessor.create();

    private void startTimer() {
//        stopped.set(false);
        mIsPaused.set(false);
        getTimerObservable().subscribeOn(Schedulers.newThread()).subscribe(recordTimeProcessor);

    }

    private Flowable<RecordTime> getTimerObservable() {
        mIsStopped.set(false);

        return Flowable.interval(1000, TimeUnit.MILLISECONDS)
                .takeWhile(tick -> !mIsStopped.get())
                .filter(timeElapsed -> !mIsPaused.get())
                .map(tick -> {
                    long seconds = mRecordTimeCounter.incrementAndGet();
                    RecordTime recordTime = new RecordTime();
                    recordTime.millis = seconds * 1000;
                    recordTime.hours = seconds / (60 * 60);
                    seconds = seconds % (60 * 60);
                    recordTime.minutes = seconds / 60;
                    seconds = seconds % 60;
                    recordTime.seconds = seconds;
                    currentRecordTime = recordTime;
                    return recordTime;
                });
    }


    private Flowable<byte[]> audioDataFlowable ;

    private final PublishProcessor<byte[]> recordDataPublishProcessor = PublishProcessor.create();

    private void startRecordThread() {

        audioDataFlowable =  Flowable.create(emitter -> {
            int bufferSize = AudioRecord.getMinBufferSize(mRecorderSampleRate, AudioFormat.CHANNEL_IN_MONO, AudioFormat.ENCODING_PCM_16BIT);
//            int bufferSize = 256;
            AudioRecord recorder = new AudioRecord(MediaRecorder.AudioSource.VOICE_RECOGNITION, mRecorderSampleRate,
                    Constants.RECORDER_CHANNELS, Constants.RECORDER_AUDIO_ENCODING, bufferSize);
            if(_isWriteFile){
                audioSaveHelper.createNewFile();

            }

            try {
                if (recorderState == RECORDER_STATE_STARTING) {
                    recorderState = RECORDER_STATE_BUSY;
                }
                recorder.startRecording ();


                do {
                    if (!mIsPaused.get()) {
                        byte[] recordBuffer = new byte[bufferSize];
                        int bytesRead = recorder.read(recordBuffer, 0, bufferSize);
                        if (bytesRead == 0) {
                            Log.e(AudioRecorder.class.getSimpleName(), "error: " + bytesRead);
                            onRecordFailure();
                        }else{
                            Log.d("kiemtra",recordBuffer.length + "");
                            emitter.onNext(recordBuffer);
                        }
                    }
                } while (recorderState == RECORDER_STATE_BUSY);
            } finally {
                recorder.stop();
                recorder.release();
                Log.d("kiemtra","release nay");
            }
            emitter.onComplete();
        }, BackpressureStrategy.DROP);
        audioDataFlowable.subscribeOn(Schedulers.io()).subscribe(recordDataPublishProcessor);
        compositeDisposable.add(recordDataPublishProcessor.onBackpressureBuffer()
                .observeOn(Schedulers.io())
                .subscribeWith(new DisposableSubscriber<byte[]>() {
                    @Override public void onNext(byte[] bytes) {
                        mDataSize += bytes.length;
                        updatePowers(bytes);



                        HashMap<String, Object> currentResult = new HashMap<>();
                        currentResult.put("duration", getDuration() * 1000);
                        currentResult.put("path", "");
                        currentResult.put("audioFormat", "mExtension");
                        currentResult.put("peakPower",mPeakPower);
                        currentResult.put("averagePower", mAveragePower);
                        currentResult.put("isMeteringEnabled", true);
                        currentResult.put("status", mStatus);
                        currentResult.put("buffer", bytes);

                        qe.add(currentResult);

                            Log.d("kiemtra",qe.size() + "");
                            if(_isWriteFile){
                                audioSaveHelper.onDataReady(bytes);

                            }
                        }



                    @Override public void onError(Throwable t) {

                    }

                    @Override public void onComplete() {
                        if(_isWriteFile) {
                            audioSaveHelper.onRecordingStopped(currentRecordTime);
                        }
                        synchronized (recorderStateMonitor) {
                            recorderState = RECORDER_STATE_IDLE;
                            recorderStateMonitor.notifyAll();
                        }
                    }
                }));
    }

    @Override public void finishRecord() {
        int recorderStateLocal = recorderState;
        mIsStopped.set(true);

        if (recorderStateLocal != RECORDER_STATE_IDLE) {
            synchronized (recorderStateMonitor) {
                recorderStateLocal = recorderState;
                if (recorderStateLocal == RECORDER_STATE_STARTING
                        || recorderStateLocal == RECORDER_STATE_BUSY) {

                    recorderStateLocal = recorderState = RECORDER_STATE_STOPPING;
                }

                do {
                    try {
                        if (recorderStateLocal != RECORDER_STATE_IDLE) {
                            recorderStateMonitor.wait();
                        }
                    } catch (InterruptedException ignore) {
                        /* Nothing to do */
                    }
                    recorderStateLocal = recorderState;
                } while (recorderStateLocal == RECORDER_STATE_STOPPING);
            }
        }


        resetRecorder();
        compositeDisposable.dispose();
        compositeDisposable = null;
        audioDataFlowable = null;
    }

    @Override public void pauseRecord() {
        qe.clear();
        mStatus = "paused";
        mPeakPower = -120;
        mAveragePower = -120;
        mIsPaused.set(true);
    }

    @Override public void resumeRecord() {
        mIsPaused.set(false);
        mStatus = "recording";

    }

    @Override public boolean isRecording() {
        return recorderState != RECORDER_STATE_IDLE;
    }

    public Flowable<byte[]> getAudioDataFlowable() {
        return recordDataPublishProcessor;
    }

    public Disposable subscribeTimer(Consumer<RecordTime> timerConsumer) {
        Disposable disposable =
                recordTimeProcessor.observeOn(AndroidSchedulers.mainThread()).subscribe(timerConsumer);
        compositeDisposable.add(disposable);
        return disposable;
    }

    boolean isPaused() {
        return mIsPaused.get();
    }

    public static class RecordTime {
        public long seconds = 0;
        public long minutes = 0;
        public long hours = 0;
        long millis = 0;
    }

    private int getDuration(){
        long duration = mDataSize / (mRecorderSampleRate * 2 * 1);
        return (int)duration;
    }


    private short[] byte2short(byte[] bData) {
        short[] out = new short[bData.length/2];
        ByteBuffer.wrap(bData).order(ByteOrder.LITTLE_ENDIAN).asShortBuffer().get(out);
        return out;
    }

    private void resetRecorder(){
        mPeakPower = -120;
        mAveragePower = -120;
        mDataSize = 0;
    }

    private void updatePowers(byte[] bdata){
        short[] data = byte2short(bdata);
        short sampleVal = data[data.length - 1];
        String[] escapeStatusList = new String[]{"paused", "stopped", "initialized", "unset"};

        if(sampleVal == 0 || Arrays.asList(escapeStatusList).contains(mStatus)){
            mAveragePower = -120; // to match iOS silent case
        }
        else {
            // iOS factor : to match iOS power level
            double iOSFactor = 0.25;
            mAveragePower = 20 * Math.log(Math.abs(sampleVal) / 32768.0)  * iOSFactor;
        }

        mPeakPower = mAveragePower;
        // Log.d(LOG_NAME, "Peak: " + mPeakPower + " average: "+ mAveragePower);
    }
}
